using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI;
using Microsoft.SharePoint.Portal.WebControls;
using Microsoft.SharePoint.Portal.WebControls.UserProfileHelper;
using Microsoft.Office.Server.UserProfiles;
using System.Diagnostics;


namespace SPC2008
{
    
    
    public class ICW : WebPart
    {
        
        #region Properties
        
        private string _commonProperties = "Office;SPS-Interests;";
         
        [Personalizable(PersonalizationScope.Shared),
        WebBrowsable(true),
        System.ComponentModel.Category("SPC2008"),
        WebDisplayName("Common Properties"),
        WebDescription("List usesr profile properties to compare. (e.g. Office;SPS-Interests)")]
        public string CommonProperties
        {
            get
            {
                return _commonProperties;
            }

            set
            {
                _commonProperties = value;
            }
        }

        #endregion


        protected override void CreateChildControls()
        {
            Debug.WriteLine("CreateChildControls");
            base.CreateChildControls();

            try
            {
                //We only work on a personal page            
                IPersonalPage page = this.Page as IPersonalPage;
                if (page.ProfilePropertyLoader != null)
                {
                    //This is the person reading the page
                    UserProfile readerProfile = page.ProfilePropertyLoader.ProfileCurrentUser;
                    //This is the person whos page is being read
                    UserProfile ownerProfile = page.ProfilePropertyLoader.ProfileLoaded;

                    //Change the Web part title based on who is viewing the part.
                    if (readerProfile.RecordId == ownerProfile.RecordId)
                    {
                        this.Title = "Your User Profile";
                    }
                    else
                    {
                        string str = (string)ownerProfile["PreferredName"].Value;
                        this.Title = string.Format("In Common with {0}", str );
                    }

                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Error in CreateChildControls");
                Debug.WriteLine(ex.Message);
            }
        }

        protected override void RenderContents(HtmlTextWriter writer)
        {
            Debug.WriteLine("CommonProperties: " + this.CommonProperties);
            Debug.WriteLine("Render Contents");
            //TODO : If it is the search account bail out.
            
            //Split the list into an array
            string[] propertiesInCommon = CommonProperties.Split(Convert.ToChar(";"));
            //Variables for Properties we read
            string readerProp = string.Empty;
            string ownerProp = string.Empty;

            Debug.WriteLine("Begin Working with Properties");
            foreach (string prop in propertiesInCommon)
            {
                Debug.WriteLine(String.Format("The property is {0}", prop));
            }

            try
            {
                //Build the DIV tag
                writer.AddAttribute(HtmlTextWriterAttribute.Class, "incommonBody");
                writer.RenderBeginTag(HtmlTextWriterTag.Div);
                writer.AddAttribute(HtmlTextWriterAttribute.Id, "icebreaker");
                writer.AddAttribute(HtmlTextWriterAttribute.Class, "ms-qrblinks");
                writer.AddAttribute(HtmlTextWriterAttribute.Border, "0");
                writer.AddAttribute(HtmlTextWriterAttribute.Cellpadding, "0");
                writer.AddAttribute(HtmlTextWriterAttribute.Cellspacing, "0");
                writer.AddAttribute(HtmlTextWriterAttribute.Width, "100%");
                writer.RenderBeginTag(HtmlTextWriterTag.Table);
        
        
                //We only work on a personal page            
                IPersonalPage page = this.Page as IPersonalPage;
                if (page.ProfilePropertyLoader != null)
                {
                    //This is the person reading the page
                    UserProfile readerProfile = page.ProfilePropertyLoader.ProfileCurrentUser;
                    //This is the person whos page is being read
                    UserProfile ownerProfile = page.ProfilePropertyLoader.ProfileLoaded;
                    //If the Reader or Owner is null something is wrong
                    if ((readerProfile != null) && (ownerProfile != null))
                    {
                        Debug.WriteLine(string.Format("Looking for properties {0} has in common with {1}...", readerProfile["PreferredName"], ownerProfile["PreferredName"]));
                        //Debug.WriteLine(string.Format("ReaderId: {0} OwnerId: {1}", readerProfile.RecordId.ToString(), ownerProfile.RecordId.ToString()));
                        
                        try
                        {
                            //Enumerate each property and compare them
                            foreach (string prop in propertiesInCommon)
                            {
                                
                                Debug.WriteLine(String.Format("Looking for match on {0}", prop));
                                string propDisplayName = readerProfile[prop].Property.DisplayName;
                                
                                try
                                {
                                    //Get the values as strings
                                    readerProp = GetPropertyAsString(readerProfile[prop]);
                                    
                                    Debug.WriteLine("reader: " + readerProp);

                                    ownerProp = GetPropertyAsString(ownerProfile[prop]);
                                    Debug.WriteLine("owner: " + ownerProp);

                                    //If either property value is blank bail out no need to process the rest.
                                    if ((ownerProp != string.Empty) && (readerProp != string.Empty))
                                    {
                                        string[] readerPropValues = readerProp.Split(Convert.ToChar(";"));
                                        string[] ownerPropValues = ownerProp.Split(Convert.ToChar(";"));

                                        //Reset Property Match
                                        string propMatch = string.Empty;
                                        int propIndex = 0;
                                        foreach (string oProp in ownerPropValues)
                                        {
                                            //If the value is blank loop
                                            if (oProp == string.Empty)
                                                continue;

                                            Debug.WriteLine(string.Format("Looking for {0} in {1}.", oProp, readerProp));
                                            propIndex = readerProp.IndexOf(oProp);
                                            if (propIndex >= 0)
                                            {
                                                switch (propMatch.Length)
                                                {
                                                    case 0:
                                                        propMatch = oProp;
                                                        break;
                                                    default:
                                                        propMatch = string.Concat(propMatch, ", ", oProp);
                                                        break;
                                                }
                                            }
                                        }
                                        if (propMatch == string.Empty)
                                        {
                                            Debug.WriteLine("Found nothing in common.");
                                        }
                                        else
                                        {
                                            Debug.WriteLine(string.Format("Found '{0}' in common", propMatch));
                                            //TODO : Add the property name.
                                            writer.Write(BuildOutput(propDisplayName, propMatch));
                                        }

                                    }
                                    else
                                    {
                                        Debug.WriteLine("One of the property fields is blank");
                                    }
                                }
                                catch (Exception ex)
                                {
                                    Debug.WriteLine(string.Format("Error getting property '{0}' Message: {1}", prop, ex.Message));
                                    //throw;
                                }
                            }

                            //writer.Write(readerProfile["SPS-Interests"].ToString() + "<br />");
                        }
                        catch
                        {
                            Debug.WriteLine("Property value is null");
                        }
                    }

                }
                else
                {
                    writer.Write("This web part works on a Personal Page");
                }
                writer.RenderEndTag();
                writer.RenderEndTag();
            }
            catch (Exception ex)
            {
                writer.Write(string.Format("An error occured: {0}", ex.Message));
                Debug.WriteLine(ex.Message);
                Debug.WriteLine(ex.StackTrace);
            }
        }

        #region Utility

        //Thanks Ishai: http://www.sharepoint-tips.com/2008/01/getting-user-profile-values-that.html
        private string GetPropertyAsString(UserProfileValueCollection upvc)
        {
            if (upvc.Count == 1)
                return upvc.Value.ToString();
            else
            {
                StringBuilder ret = new StringBuilder("");
                UserProfileValueCollection values = upvc;
                System.Collections.IEnumerator allValues = values.GetEnumerator();
                while (allValues.MoveNext())
                {
                    ret.Append(allValues.Current.ToString());
                    ret.Append(";");
                }
                return ret.ToString();
            }
        }

        private string BuildOutput(string propertyName, string propertyValue)
        {
            return string.Format("<tr><td><span style=\"color: black\">{0}:</span></td><td>{1}</td></tr>", propertyName, propertyValue);
        }

        #endregion
    }
}
